from SCons.Script import *

def addGettextBuilder(env):

	GettextAction = SCons.Action.Action("$GETTEXTCOM", "$GETTEXTCOMSTR")
	
	env["GETTEXT"]    = env.Detect("msgfmt")
	env["GETTEXTCOM"] = "$GETTEXT --check-format --check-domain -f -o $TARGET $SOURCE"
	
	builder = Builder(
		action = GettextAction,
		suffix = '.mo',
		src_suffix = '.po',
		single_source = True)

	env.Append(BUILDERS = {'Gettext': builder})

	XgettextAction = SCons.Action.Action("$XGETTEXTCOM", "$XGETTEXTCOMSTR")
	
	env["XGETTEXT"]    = env.Detect("xgettext")
	env["XGETTEXTCOM"] = "$XGETTEXT --keyword=_ --from-code utf-8 --package-name=gpick $XGETTEXT_FLAGS --output=$TARGET $SOURCES"
	
	builder = Builder(
		action = XgettextAction,
		suffix = '.pot',
		src_suffix = '.cpp',
		single_source = False)

	env.Append(BUILDERS = {'Xgettext': builder})

	MsgmergeAction = SCons.Action.Action("$MSGMERGECOM", "$MSGMERGECOMSTR")
	
	env["MSGMERGE"]    = env.Detect("msgmerge")
	env["MSGMERGECOM"] = "$MSGMERGE $MSGMERGE_FLAGS --output-file=$TARGET $SOURCES"
	
	builder = Builder(
		action = MsgmergeAction,
		suffix = '.pot',
		src_suffix = '.pot',
		single_source = False)

	env.Append(BUILDERS = {'Msgmerge': builder})

	MsgcatAction = SCons.Action.Action("$MSGCATCOM", "$MSGCATCOMSTR")
	
	env["MSGCAT"]    = env.Detect("msgcat")
	env["MSGCATCOM"] = "$MSGCAT $MSGCAT_FLAGS --output-file=$TARGET $SOURCES"
	
	builder = Builder(
		action = MsgcatAction,
		suffix = '.pot',
		src_suffix = '.pot',
		single_source = False)

	env.Append(BUILDERS = {'Msgcat': builder})




from SCons.Script import *
import re
def addTemplateBuilder(env):
	def buildEmitter(target, source, env):
		data = open(str(File(source[0]).srcnode())).read()
		dict = env.Dictionary()
		keys = dict.keys()
		for key in keys:
			if isinstance(dict[key], str):
				env.Depends(target, env.Value(dict[key]))
		return target, source
	def buildFile(target, source, env):
		source_dest = str(target[0])
		wfile = open(source_dest, "w")
		data = open(str(File(source[0]).srcnode())).read()
		dict = env.Dictionary()
		keys = dict.keys()
		for key in keys:
			if isinstance(dict[key], str):
				data = re.sub("%" + key + "%", dict[key], data)
		wfile.write(data)
		wfile.close()
		return 0
	def buildString(target, source, env):
		return "Preparing file %s" % os.path.basename(str(target[0]))
	builder = Builder(
		action = SCons.Action.Action(buildFile, buildString),
		emitter = buildEmitter,
	)
	env.Append(BUILDERS = {'Template': builder})



from SCons.Script import *

def addFlexBuilder(env):

	FlexAction = SCons.Action.Action("$FLEXCOM", "$FLEXCOMSTR")
	
	env["FLEX"]      = env.Detect("flex")
	env["FLEXCOM"] = "flex --header-file=${TARGET.base}.h -o $TARGET $SOURCE"
	
	def headerEmitter(target, source, env): 
		bs = SCons.Util.splitext(str(source[0].name))[0] 
		target.append(bs + '.h') 
		return (target, source) 
	
	builder = Builder(action = FlexAction,
		suffix = '.cpp',
		src_suffix = '.l',
		emitter = headerEmitter)

	env.Append(BUILDERS = {'Flex': builder})



#!/usr/bin/env python
import os
import time
import SCons
import re
import string
import glob
import subprocess
from lemon import *
from flex import *
from gettext import *
from resource_template import *
from ragel import *
from template import *
from SCons.Script import *
from SCons.Util import *
from SCons.Script.SConscript import SConsEnvironment
import SCons.Script.SConscript
import SCons.SConf
import SCons.Conftest

def MatchFiles (files, path, repath, dir_exclude_pattern,  file_exclude_pattern):
	for filename in os.listdir(path):
		fullname = os.path.join (path, filename)
		repath_file =  os.path.join (repath, filename);
		if os.path.isdir (fullname):
			if not dir_exclude_pattern.search(repath_file):
				MatchFiles (files, fullname, repath_file, dir_exclude_pattern, file_exclude_pattern)
		else:
			if not file_exclude_pattern.search(filename):
				files.append (fullname)

def CheckPKG(context, name):
	context.Message('Checking for library %s... ' % name)
	ret = context.TryAction('pkg-config --exists "%s"' % name)[0]
	context.Result(ret)
	return ret

def CheckProgram(context, env, name, member_name):
	context.Message('Checking for program %s... ' % name)
	if env[member_name]:
		context.Result(True)
		return True
	else:
		context.Result(False)
		return False

def CompareVersions(a, b):
	for i in range(0, min(len(a), len(b))):
		if a[i] < b[i]:
			return 1
		if a[i] > b[i]:
			return -1
	return 0

def CheckBoost(context, version):
	context.Message('Checking for library boost >= %s... ' % (version))
	result, boost_version = context.TryRun("""
#include <boost/version.hpp>
#include <iostream>
int main(){
std::cout << BOOST_LIB_VERSION << std::endl;
return 0;
}
""", '.cpp')
	if result:
		found_version = boost_version.strip('\r\n\t ').split('_')
		required_version = version.strip('\r\n\t ').split('.')
		result = CompareVersions(required_version, found_version) >= 0
	context.Result(result)
	return result

class GpickLibrary(NodeList):
	include_dirs = []

class GpickEnvironment(SConsEnvironment):
	extern_libs = {}
	def AddCustomBuilders(self):
		addLemonBuilder(self)
		addFlexBuilder(self)
		addGettextBuilder(self)
		addResourceTemplateBuilder(self)
		addTemplateBuilder(self)
		addRagelBuilder(self)
		
	def DefineLibrary(self, library_name, library):
		self.extern_libs[library_name] = library
		
	def UseLibrary(self, library_name):
		lib = self.extern_libs[library_name]
		
		for i in lib:
			lib_include_path = os.path.split(i.path)[0]
			self.PrependUnique(LIBS = [library_name], LIBPATH = ['#' + lib_include_path])
			
		self.PrependUnique(CPPPATH = lib.include_dirs)
		
		return lib

	def ConfirmPrograms(self, conf, programs):
		conf.AddTests({'CheckProgram': CheckProgram})
		
		for evar, args in programs.iteritems():
			found = False
			for name, member_name in args['checks'].iteritems():
				if conf.CheckProgram(self, name, member_name):
					found = True;
					break
			if not found:
				if 'required' in args:
					if not args['required']==False:
						self.Exit(1)
				else:
					self.Exit(1)

	def ConfirmLibs(self, conf, libs):
		conf.AddTests({'CheckPKG': CheckPKG})
		
		for evar, args in libs.iteritems():
			found = False
			for name, version in args['checks'].iteritems():
				if conf.CheckPKG(name + ' ' + version):
					self[evar]=name
					found = True;
					break
			if not found:
				if 'required' in args:
					if not args['required']==False:
						self.Exit(1)
				else:
					self.Exit(1)

	def ConfirmBoost(self, conf, version):
		conf.AddTests({'CheckBoost': CheckBoost})
		if conf.CheckBoost(version):
			return
		else:
			self.Exit(1)

	def InstallPerm(self, dir, source, perm):
		obj = self.Install(dir, source)
		for i in obj:
			self.AddPostAction(i, Chmod(i, perm))
		return dir

	def InstallPermAutoDir(self, dir, relative_dir, source, perm):
		for f in Flatten(source):
			path = dir
			if str(f.get_dir()).startswith(relative_dir):
				path = os.path.join(path, str(f.get_dir())[len(relative_dir):])
			else:
				path = os.path.join(path, str(f.get_dir()))
			obj = self.Install(path, f)
			for i in obj:
				self.AddPostAction(i, Chmod(i, perm))
		return dir

	InstallProgram = lambda self, dir, source: GpickEnvironment.InstallPerm(self, dir, source, 0755)
	InstallData = lambda self, dir, source: GpickEnvironment.InstallPerm(self, dir, source, 0644)
	InstallDataAutoDir = lambda self, dir, relative_dir, source: GpickEnvironment.InstallPermAutoDir(self, dir, relative_dir, source, 0644)

	def GetSourceFiles(self, dir_exclude_pattern, file_exclude_pattern):
		dir_exclude_prog = re.compile(dir_exclude_pattern)
		file_exclude_prog = re.compile(file_exclude_pattern)
		files = []
		MatchFiles(files, self.GetLaunchDir(), os.sep, dir_exclude_prog, file_exclude_prog)
		return files

	def GetVersionInfo(self):
		try:
			revision = subprocess.Popen(['git', 'show', '--no-patch', '--format="%H %ct"'], shell=False, stdout=subprocess.PIPE, stderr=subprocess.STDOUT).communicate()[0]
			match = re.search('([\d\w]+) (\d+)', str(revision))
			rev_hash = match.group(1)
			commit_date = time.gmtime(int(match.group(2)))
			rev_date = time.strftime("%Y-%m-%d", commit_date)
			rev_time = time.strftime("%H:%M:%S", commit_date)
		except:
			try:
				with open("../version.txt", "r") as version_file:
					lines = version_file.read().splitlines()
					rev_hash = lines[0]
					rev_date = lines[1]
					rev_time = lines[2]
			except:
				rev_hash = 'unknown'
				commit_date = time.gmtime()
				rev_date = time.strftime("%Y-%m-%d", commit_date)
				rev_time = time.strftime("%H:%M:%S", commit_date)
		self.Replace(
			GPICK_BUILD_REVISION = rev_hash,
			GPICK_BUILD_DATE = rev_date,
			GPICK_BUILD_TIME = rev_time,
		);

def RegexEscape(str):
	return str.replace('\\', '\\\\')

def WriteNsisVersion(target, source, env):
	for t in target:
		for s in source:
			file = open(str(t),"w")
			file.writelines('!define VERSION "' + str(env['GPICK_BUILD_VERSION']) + '"')
			file.close()
	return 0

def Glob(path):
	files = []
	for f in glob.glob(os.path.join(path, '*')):
		if os.path.isdir(str(f)):
			files.extend(Glob(str(f)));
		else:
			files.append(str(f));
	return files





from SCons.Script import *
from SCons.Tool.install import copyFunc

def addLemonBuilder(env):
	
	LemonAction = SCons.Action.Action("$LEMONCOM", "$LEMONCOMSTR")
	
	env["LEMON"]      = env.Detect("lemon")
	env["LEMONCOM"] = "cd build/; $LEMON $SOURCE"
	
	def headerEmitter(target, source, env): 
		bs = SCons.Util.splitext(str(source[0].name))[0] 
		target.append(bs + '.h')
		bs = SCons.Util.splitext(str(target[0]))[0] + ".y"
		source.append(bs) 	
		return (target, source)
		
	def buildAction(target, source, env):
		source_dest = SCons.Util.splitext(str(target[0]))[0] + ".y"
		Execute(Copy('build/lempar.c', 'extern/lempar.c'))
		Execute(Copy(source_dest, File(source[0]).srcnode()))
		LemonAction.execute([target], ['../' + source_dest], env);
		Execute(Delete(source_dest))
		return 0
	
	builder = Builder(
		action = buildAction,
		suffix = '.c',
		src_suffix = '.y',
		emitter = headerEmitter)
		
	env.Append(BUILDERS = {'Lemon': builder})



import os,sys,re
parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0,parentdir) 
import version

def getversion(style):
	if style == "full":
		print version.GPICK_BUILD_VERSION
	else:
		result = re.match(r"^(\d+\.\d+\.\d+)(.*)$", version.GPICK_BUILD_VERSION)
		if result:
			print result.group(1) 
		else:
			print "invalid_version"



#!/usr/bin/env python
from SCons.Script import *
from SCons.Script.SConscript import SConsEnvironment
import SCons.Script.SConscript

def addRagelBuilder(env):
	RagelAction = SCons.Action.Action("$RAGELCOM", "$RAGELCOMSTR")
	env["RAGEL"] = env.Detect("ragel")
	env["RAGELCOM"] = "$RAGEL $RAGELFLAGS -o $TARGET $SOURCE"
	env["RAGELFLAGS"] = "-F1 -C"
	builder = Builder(
		action = RagelAction,
		suffix = '.cpp',
		src_suffix = '.rl',
		single_source = True)
	env.Append(BUILDERS = {'Ragel': builder})



from SCons.Script import *
import re
def addResourceTemplateBuilder(env):
	def buildResourceFile(target, source, env):
		source_dest = SCons.Util.splitext(str(target[0]))[0] + ".rc"
		wfile = open(source_dest,"w")
		data = open(str(File(source[0]).srcnode())).read()
		for key, var in env['RESOURCE_TEMPLATE_VARS'].iteritems():
			data = re.sub("%" + key + "%", var, data)
		wfile.write(data)
		wfile.close()
		return 0
	def buildResourceFileString(target, source, env):
		return "Preparing resource file %s" % os.path.basename(str(target[0]))
	builder = Builder(
		action = SCons.Action.Action(buildResourceFile, buildResourceFileString),
		suffix = '.rc',
		src_suffix = '.rct',
	)
	env.Append(BUILDERS = {'ResourceTemplate': builder})
